@@ -4,44 +4,45 @@ extern crate tokio_core;
 extern crate futures;
 
 use std::thread;
+use std::io::Read;
 use std::io::Write;
 use std::io::SeekFrom;
 use std::net::TcpListener;
+use std::time::Duration;
+
+use futures::Sink;
+use futures::sync::mpsc;
 
 use file_api::http_reader::HttpReader;
 use file_api::reader::Reader;
 
 fn mock_server(port: &str, messages: Vec<String>, tester: &mut FnMut()) {
+
+  let mut responses = messages.clone();
+  responses.reverse();
+
   let address = ["127.0.0.1:", port].join("");
   let server = TcpListener::bind(address).unwrap();
 
-  let mut responses = messages.clone();
-
+  let (mut tx, _rx) = mpsc::channel(messages.len());
   let handler = thread::spawn(move || {
     while responses.len() != 0 {
-      for server_stream in server.incoming() {
-        match server_stream {
-          Ok(mut stream) => {
-            let remove_first =
-              match responses.first() {
-                Some(response) => {
-                  stream.write_all(response.as_str().as_bytes()).unwrap();
-                  true
-                },
-                None => {
-                  false
-                },
-              };
+      match responses.pop() {
+        Some(response) => {
+          let mut inc = server.accept().unwrap().0;
+          inc.set_read_timeout(Some(Duration::from_secs(5))).unwrap();
+          inc.set_write_timeout(Some(Duration::from_secs(5))).unwrap();
 
-            if remove_first {
-              responses.remove(0);
-              break;
-            }
+          let mut message = "".to_string();
+          let _result = inc.read_to_string(&mut message);
+          // println!("{:?} ==> {:?}", message, response);
+
+          if message != "" {
+            inc.write_all(response.as_ref()).unwrap();
+            let _ = tx.start_send(());
           }
-          Err(e) => {
-            println!("Unable to connect: {}", e);
-          }
-        }
+        },
+        None => {}
       }
     }
   });
@@ -103,6 +104,7 @@ fn http_read_data() {
     "HTTP/1.1 200 OK\r\nContent-Length: 4\r\nContent-Range: bytes 0-4/19\r\n\r\nsome".to_string(),
     "HTTP/1.1 200 OK\r\nContent-Length: 4\r\nContent-Range: bytes 4-8/19\r\n\r\ndata".to_string(),
   ];
+
   fn check() {
     let filename = "http://127.0.0.1:8883/data".to_string();
     let mut reader : HttpReader = Reader::open(&filename);
